﻿/*
    Copyright 2002-2014 KeyDays LLC.

    This file is a part of KeyDays.Lib.
    KeyDays.Lib is licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. 
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.IO;
using System.Management;
using System.Runtime.InteropServices;
using System.Reflection;
using System.ComponentModel;
using Microsoft.Win32;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace KeyDays.Lib.Util
{
    public static class ToolSet
    {
        #region fields
        public delegate void BeforeFileCopyHandler(string sourcePath, string targetPath);
        #endregion fields

        #region methods

        public static string GetMACAddress()
        {
            ManagementClass mc = new ManagementClass("Win32_NetworkAdapterConfiguration");
            ManagementObjectCollection moc = mc.GetInstances();
            string MACAddress = String.Empty;
            foreach (ManagementObject mo in moc)
            {
                if (true == (bool)mo["IPEnabled"])
                {
                    MACAddress = mo["MacAddress"].ToString();
                    break;
                }
                mo.Dispose();
            }
            return MACAddress;
        }

        public static bool CreateDirectoryIfMissing(string path)
        {
            if (true == System.IO.Directory.Exists(path))
            {
                return false;
            }
            //KeyDays.Lib.Diagnostics.Trace.TraceWarning(string.Format(@"Directory did not exist: {0}", path));
            System.IO.Directory.CreateDirectory(path);
            //KeyDays.Lib.Diagnostics.Trace.TraceInformation(string.Format(@"Created the new directory: {0}", path));
            return true;
        }

        public static void CopyDirectory(string sourceBasePath, string targetBasePath, string searchPattern, SearchOption searchOption)
        { 
            CopyDirectory(sourceBasePath, targetBasePath, searchPattern, searchOption, null);
        }

        //--- http://msdn.microsoft.com/en-us/library/cc148994.aspx     
        public static void CopyDirectory(string sourceBasePath, string targetBasePath, string searchPattern, SearchOption searchOption, BeforeFileCopyHandler beforeFileCopyHandler)
        {
            sourceBasePath = Path.GetFullPath(sourceBasePath);
            targetBasePath = Path.GetFullPath(targetBasePath);

            if (false == sourceBasePath.EndsWith(@"\"))
            {
                sourceBasePath += @"\";
            }

            if (false == targetBasePath.EndsWith(@"\"))
            {
                targetBasePath += @"\";
            }

            if (false == Directory.Exists(sourceBasePath))
            {
                return;
            }

            string
                TargetFileName = null,
                TargetPath = null;

            string[] files = Directory.GetFiles(sourceBasePath, searchPattern, searchOption);

            // Copy the files and overwrite destination files if they already exist.
            foreach (string SourceFileName in files)
            {
                // Use static Path methods to extract only the file name from the path.
                // fileName = Path.GetFileName(s);
                // sourcePath = Path.GetDirectoryName(s);
                TargetFileName = SourceFileName.Replace(sourceBasePath, targetBasePath);
                if(false == String.IsNullOrEmpty(TargetPath = Path.GetDirectoryName(TargetFileName.Replace(Path.GetFileName(SourceFileName), ""))))
                {
                    CreateDirectoryIfMissing(TargetPath);
                }
                if (null != beforeFileCopyHandler)
                {
                    beforeFileCopyHandler.Invoke(SourceFileName, TargetFileName);
                }
                //--- http://blogs.msdn.com/b/bclteam/archive/2007/02/13/long-paths-in-net-part-1-of-3-kim-hamilton.aspx
                if (KeyDays.Lib.WindowsAPI.Kernel32Wrapper.MAX_PATH > TargetFileName.Length && KeyDays.Lib.WindowsAPI.Kernel32Wrapper.MAX_PATH > SourceFileName.Length)
                {
                    File.Copy(SourceFileName, TargetFileName, true);
                    continue;
                }

                TargetFileName = @"\\?\" + TargetFileName;
                KeyDays.Lib.WindowsAPI.Kernel32Wrapper.CopyFile(SourceFileName, TargetFileName, false);
            }
        }

        public static long GetDiskFreeSpace(string driveName)
        {
            return long.Parse(ToolSet.GetDiskInfo(driveName, "FreeSpace"));
        }

        //--- http://stackoverflow.com/questions/1393711/c-get-free-disk-space
        public static string GetDiskInfo(string driveName, string key)
        {         
            string strPath = String.Format(@"win32_logicaldisk.deviceid=""{0}""", driveName.Replace(@"\", ""));
            ManagementObject objManagementObject = new ManagementObject(strPath);
            objManagementObject.Get();
            return objManagementObject[key].ToString();
        }

        public static string Escape(string xmlString)
        {
            if (true == string.IsNullOrEmpty(xmlString))
            {
                return null;
            }
            return System.Security.SecurityElement.Escape(xmlString);
        }

        public static string UnescapeXML(string xmlString)
        {
            if (true == string.IsNullOrEmpty(xmlString))
            {
                return null;
            }
            return xmlString.Replace("&apos;", "'").Replace("&quot;", "\"").Replace("&gt;", ">").Replace("&lt;", "<").Replace("&amp;", "&");
        }

        //--- http://msdn.microsoft.com/en-us/library/system.componentmodel.descriptionattribute%28v=vs.110%29.aspx
        public static string GetDescription(Enum enumType)
        {
            MemberInfo[] MemberInfo = enumType.GetType().GetMember(enumType.ToString());

            if (0 == MemberInfo.Length)
            {
                return enumType.ToString();
            }

            var Attribute = MemberInfo[0].GetCustomAttributes(typeof(DescriptionAttribute), false).FirstOrDefault();

            if (null == Attribute)
            {
                return MemberInfo[0].Name;
            }

            return ((DescriptionAttribute)Attribute).Description;
        }

        //--- http://stackoverflow.com/questions/58510/using-net-how-can-you-find-the-mime-type-of-a-file-based-on-the-file-signature
        public static string GetMimeType(string fileExtention, string ifNotFoundUseThis = "application/octet-stream")
        {
            // string MimeType = "application/unknown";

            RegistryKey 
                objRegistryKey = Registry.ClassesRoot.OpenSubKey(fileExtention.ToLower());

            Object 
                ContentType = null;

            if (null == objRegistryKey || null == (ContentType = objRegistryKey.GetValue("Content Type")))
            {
                return ifNotFoundUseThis;
            }

            return ContentType.ToString();
        }

        public static string AssemblyGuidString
        {
            get { return ToolSet.GetAssemblyGuidString(System.Reflection.Assembly.GetEntryAssembly()); }
        }

        public static string GetAssemblyGuidString(System.Reflection.Assembly assembly)
        {
            object[] objects = assembly.GetCustomAttributes(typeof(System.Runtime.InteropServices.GuidAttribute), false);
            if (0 == objects.Length)
            {
                return String.Empty;
            }

            return ((System.Runtime.InteropServices.GuidAttribute)objects[0]).Value;
        }

        public static string GetFolderName(string selectedPath = null, string description = "Select a folder", bool showNewFolderButton = true)
        {
            FolderBrowserDialog
                objFolderBrowserDialog = new FolderBrowserDialog();

            objFolderBrowserDialog.SelectedPath = selectedPath;
            objFolderBrowserDialog.Description = description;
            objFolderBrowserDialog.ShowNewFolderButton = showNewFolderButton;

            if (DialogResult.OK == objFolderBrowserDialog.ShowDialog())
            {
                return objFolderBrowserDialog.SelectedPath;
            }

            return selectedPath;
        }

        public static string NormalizeFolderName(string folderName, bool appendSlash = true)
        {
            string
                AdditionalPath = Regex.Replace(folderName, "^{.*}", ""),
                SpecialFolder = folderName.Replace(AdditionalPath, "").Replace("{", "").Replace("}", ""),
                BaseFolder = null;

            if (false == String.IsNullOrWhiteSpace(SpecialFolder))
            {
                BaseFolder = "_AssemblyDirectory_" == SpecialFolder
                            ? SystemInfo.AssemblyDirectory
                            : Environment.GetFolderPath((Environment.SpecialFolder)Enum.Parse(typeof(Environment.SpecialFolder), SpecialFolder));

                if (true == String.IsNullOrWhiteSpace(BaseFolder))
                {
                    throw new KeyDays.Lib.Exceptions.SYSTEMException(String.Format("{0} has been resolved to an empty string.", SpecialFolder));
                }

                if (false == Directory.Exists(BaseFolder))
                {
                    throw new KeyDays.Lib.Exceptions.SYSTEMException(String.Format("{0} has been resolved to {1} and {1} folder does not exists.", SpecialFolder, BaseFolder));
                }

                folderName = String.Format("{0}{1}", BaseFolder, AdditionalPath);
            }

            bool
                NetworkFolder = folderName.StartsWith(@"\");

            folderName = Regex.Replace(folderName, @"\\+", "<");
            folderName = Regex.Replace(folderName, @"<", @"\");
            folderName = String.Format("{0}{1}", NetworkFolder ? @"\" : "", folderName);

            if (true == appendSlash && false == folderName.EndsWith(@"\"))
            {
                folderName += @"\";
            }

            return NetworkFolder ? folderName : Path.GetFullPath(folderName);
        }
        
        public static DateTime UnixToWindowsDateTime(string unixTimestamp)
        {
            return ToolSet.UnixToWindowsDateTime(double.Parse(unixTimestamp));
        }

        public static DateTime UnixToWindowsDateTime(double unixTimestamp)
        {
            return (new DateTime(1970, 1, 1, 0, 0, 0)).AddSeconds(unixTimestamp);
        }

        public static double WindowsToUnixDateTime(DateTime windowsDateTime)
        {
            return (double)(windowsDateTime.Subtract(new DateTime(1970, 1, 1)).TotalSeconds);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="stringMap">
        /// <code>
        /// @"
        ///     Key1	>	Value1
        ///     Key2	>	Value2
        /// "            
        /// </code>
        /// </param>
        /// <returns>
        /// Dictionary.Key: String left to >
        /// Dictionary.Value: String right to >
        /// </returns>
        public static Dictionary<string, string> MapToLookUp(string stringMap)
        {
            string[]
               Tokens = Regex.Split(stringMap, @"\r\n|\r|\n|>")
                       .Select(token => token.Trim())
                       .Where(token => false == String.IsNullOrWhiteSpace(token)).ToArray();

            return Tokens.Take(Tokens.Length - 1).Select((value, index) => index).Where(index => 0 == index % 2)
                                   .ToDictionary(index => Tokens[index], index => Tokens[index + 1]);
        }
        #endregion methods
    }
}